<?php
/* --------------------------------------------------------------
   OvistoCouponFactory.inc.php 2018-01-31
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class OvistoCouponFactory
 *
 * @category   Core
 * @package    Ovisto
 * @subpackage Factories
 */
class OvistoCouponFactory
{
    /**
     * @var OvistoLogoStorageWriterInterface $logoStorageWriter
     */
    protected $logoStorageWriter;
    
    /**
     * @var OvistoCouponCodeReaderInterface $couponCodeReader
     */
    protected $couponCodeReader;
    
    
    /**
     * OvistoCouponFactory constructor.
     *
     * @param OvistoLogoStorageWriterInterface $logoStorageWriter
     * @param OvistoCouponCodeReaderInterface  $couponCodeReader
     */
    public function __construct(
        OvistoLogoStorageWriterInterface $logoStorageWriter,
        OvistoCouponCodeReaderInterface $couponCodeReader
    ) {
        $this->logoStorageWriter = $logoStorageWriter;
        $this->couponCodeReader  = $couponCodeReader;
    }
    
    
    /**
     * Creates a coupon instance from JSON decoded coupon data.
     *
     * @param stdClass $coupon
     *
     * @return OvistoCoupon
     */
    public function createFromJson(stdClass $coupon)
    {
        return MainFactory::create(OvistoCoupon::class,
            $this->logoStorageWriter,
            $this->couponCodeReader,
            new IdType($coupon->id),
            new StringType($coupon->name),
            new StringType($coupon->description),
            new StringType($coupon->campaign_url),
            new StringType($coupon->partner_title),
            new StringType($coupon->offer_img),
            new StringType($coupon->type));
    }
    
    
    /**
     * Creates a collection of coupon instances.
     *
     * @param array $couponArray
     *
     * @return OvistoCouponCollection
     */
    public function createCollectionFromArray(array $couponArray)
    {
        return MainFactory::create(OvistoCouponCollection::class, $couponArray);
    }
}