<?php
/* --------------------------------------------------------------
   OvistoCouponService.inc.php 2018-01-26
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class OvistoCouponService
 */
class OvistoCouponService
{
    /**
     * @var OvistoCouponRepositoryInterface $couponRepository
     */
    protected $couponRepository;
    
    
    /**
     * OvistoService constructor.
     *
     * @param OvistoCouponRepositoryInterface $couponRepository
     */
    public function __construct(OvistoCouponRepositoryInterface $couponRepository)
    {
        $this->couponRepository = $couponRepository;
    }
    
    
    /**
     * Gets a coupon by its ID.
     * 
     * @param IdType $id
     *
     * @return OvistoCoupon
     */
    public function getCoupon(IdType $id)
    {
        return $this->couponRepository->getCouponById($id);
    }
    
    
    /**
     * Gets all Ovisto coupons.
     *
     * @param IntType $limit
     * @param IntType $offset
     * 
     * @return OvistoCouponCollection
     */
    public function getCoupons(IntType $limit, IntType $offset)
    {
        return $this->couponRepository->getAllCoupons($limit, $offset);
    }
    
    
    /**
     * Gets the total number of available Ovisto coupons.
     * 
     * @return int
     */
    public function getCouponQuantity()
    {
        return $this->couponRepository->getCouponQuantity();
    }
}