<?php
/* --------------------------------------------------------------
   dependent.inc.php 2018-03-02
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/** @var \DatabaseModel $this */

/**
 * Add tables for admin access
 */
$t_check                    = $this->table_exists('admin_access_group_items');
$adminAccessAlreadyImported = $t_check;
if(!$t_check)
{
	$t_query   = "
		CREATE TABLE `admin_access_group_items` (
			`admin_access_group_id` INT(11) UNSIGNED NOT NULL DEFAULT '0',
			`identifier` varchar(255) NOT NULL DEFAULT '',
			`type` ENUM('CONTROLLER', 'PAGE', 'AJAX_HANDLER') NOT NULL DEFAULT 'CONTROLLER',
			PRIMARY KEY (`identifier`, `type`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
	";
	$t_success &= $this->query($t_query);
}

$t_check = $this->table_exists('admin_access_group_descriptions');
if(!$t_check)
{
	$t_query   = "
		CREATE TABLE `admin_access_group_descriptions` (
			`admin_access_group_id` INT(11) UNSIGNED NOT NULL DEFAULT '0',
			`language_id` int(11) NOT NULL DEFAULT '1',
			`name` varchar(255) NOT NULL DEFAULT '',
			`description` text NOT NULL,
			PRIMARY KEY (`admin_access_group_id`, `language_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
	";
	$t_success &= $this->query($t_query);
}

$t_check = $this->table_exists('admin_access_groups');
if(!$t_check)
{
	$t_query   = "
		CREATE TABLE `admin_access_groups` (
			`admin_access_group_id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
			`parent_id` INT(11) UNSIGNED NOT NULL DEFAULT '0',
			`sort_order` INT(3) NOT NULL DEFAULT '0',
			`protected` SMALLINT UNSIGNED NOT NULL DEFAULT '0',
			PRIMARY KEY (`admin_access_group_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
	";
	$t_success &= $this->query($t_query);
	
	$sortOrder = 1;
	
	/**
	 * ---------- API ----------
	 */
	$apiGroupId = $this->addAdminAccessGroup(array(1 => 'API', 2 => 'API'), array(
		1 => 'Allows the general usage of the shop API. This permission may be necessary for some inventory management system.',
		2 => 'Erlaubt die generelle Nutzung der API. Dies wird z.B. zur Nutzung einiger Warenwirtschaftssysteme benötigt.',
	), $sortOrder++);
	if($apiGroupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($apiGroupId, 'CONTROLLER', array('DefaultApiV2', 'HttpApiV2'));
	}
	else
	{
		$t_success = false;
	}
	
	/**
	 * ---------- Gambio Admin Web UI ----------
	 */
	$groupId = $this->addAdminAccessGroup(array(1 => 'Gambio Admin Web UI', 2 => 'Gambio Admin Web UI'), array(
		1 => 'Allows the general usage of the Gambio Admin web interface and access to the Gambio news.',
		2 => 'Erlaubt die generelle Nutzung der Gambio Admin Weboberfläche und Zugriff auf die Gambio News.',
	), $sortOrder++);
	if($groupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($groupId, 'AJAX_HANDLER', array(
			'AdminInfobox',
			'GambioUpdate',
			'LoadAdminInfoBoxes',
			'LoadNews'
		));
		$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
			'Dashboard',
			'UserConfiguration',
		));
		$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
			'start.php',
			'admin.php',
			'request_port.php'
		));
	}
	else
	{
		$t_success = false;
	}
	
	/**
	 * ---------- Orders ----------
	 */
	$ordersGroupId = $this->addAdminAccessGroup(array(1 => 'Orders', 2 => 'Bestellungen'), array(
		1 => 'Allows the management of the orders.',
		2 => 'Erlaubt die Verwaltung von Bestellungen.',
	), $sortOrder++);
	if($ordersGroupId === false)
	{
		$t_success = false;
	}
	
	if($ordersGroupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($ordersGroupId, 'CONTROLLER', array(
			'Dashboard/getLatestOrders',
			'OrdersApiV2',
			'OrdersHistoryApiV2',
			'OrdersItemsApiV2',
			'OrdersItemsAttributesApiV2',
			'OrdersModalsAjax',
			'OrdersOverview',
			'OrdersOverviewAjax',
			'OrderStatusesApiV2',
			'OrdersTotalsApiV2',
			'OrderTooltip'
		));
		$t_success &= $this->addAdminAccessGroupItem($ordersGroupId, 'PAGE', array(
			'gm_order_menu.php',
			'gm_send_order.php',
			'mobile.php',
			'orders.php',
			'orders_actions.php',
			'orders_edit.php',
			'orders_edit_options.php',
			'orders_edit_other.php',
			'orders_edit_products.php',
			'orders_status.php',
			'popup_memo.php',
		));
		
		/**
		 * ---------- Withdrawals ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Withdrawals', 2 => 'Widerrufe'), array(
			1 => 'Allows the access to the withdrawals.',
			2 => 'Erlaubt den Zugriff auf die Widerrufe.',
		), $sortOrder++, $ordersGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'withdrawals.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Invoices ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Invoices', 2 => 'Rechnungen'), array(
			1 => 'Allows the access to the invoices.',
			2 => 'Erlaubt den Zugriff auf die Rechnungen.',
		), $sortOrder++, $ordersGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'InvoicesModalsAjax',
				'InvoicesOverview',
				'InvoicesOverviewAjax'
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array('gm_pdf_order.php'));
		}
		else
		{
			$t_success = false;
		}
	}
	
	/**
	 * ---------- Customers ----------
	 */
	$customerGroupId = $this->addAdminAccessGroup(array(1 => 'Customers', 2 => 'Kunden'), array(
		1 => 'Allows the management of the customers.',
		2 => 'Erlaubt die Verwaltung von Kunden.',
	), $sortOrder++);
	if($customerGroupId === false)
	{
		$t_success = false;
	}
	
	if($customerGroupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($customerGroupId, 'CONTROLLER', array(
			'AddressesApiV2',
			'AddressFormatsApiV2',
			'AttachmentsApiV2',
			'CustomersApiV2',
		));
		$t_success &= $this->addAdminAccessGroupItem($customerGroupId, 'PAGE', array(
			'create_account.php',
			'customers.php',
			'gm_emails.php',
			'gm_emails_preview.php',
		));
		
		/**
		 * ---------- Customer groups ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Customer Groups', 2 => 'Kundengruppen'), array(
			1 => 'Allows the management of the customer groups.',
			2 => 'Erlaubt die Verwaltung von Kundengruppen.',
		), $sortOrder++, $customerGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'customers_status.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Emails ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Emails', 2 => 'E-Mails'), array(
			1 => 'Allows the management of the emails.',
			2 => 'Erlaubt die Verwaltung von E-Mails.',
		), $sortOrder++, $customerGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'Emails',
				'EmailsApiV2',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'mail.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Admin access management ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Roles and Permissions', 2 => 'Rollen und Berechtigungen'),
		                                      array(
			                                      1 => 'Allows the management of the admin access permissions.',
			                                      2 => 'Erlaubt die Verwaltung von Zugriffsrechten.',
		                                      ), $sortOrder++, $customerGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'AdminAccess',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'accounting.php',
			));
		}
		else
		{
			$t_success = false;
		}
	}
	
	/**
	 * ---------- Google ----------
	 */
	$groupId = $this->addAdminAccessGroup(array(1 => 'Google', 2 => 'Google'), array(
		1 => 'Allows the usage of the Google service.',
		2 => 'Erlaubt die Nutzung des Google Services. ',
	), $sortOrder++);
	if($groupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
			'GoogleShopping',
			'GoogleShoppingAjax',
			'AdWordsCampaignsOverviewAjax',
			'GoogleAdWords',
		));
	}
	else
	{
		$t_success = false;
	}
	
	/**
	 * ---------- Catalog ----------
	 */
	$catalogGroupId = $this->addAdminAccessGroup(array(1 => 'Catalog', 2 => 'Artikel'), array(
		1 => 'Allows the management of products, categories, special prices, attributes, properties and product filters.',
		2 => 'Erlaubt die Verwaltung von Artikeln, Kategorien, Sonderangeboten, Eigenschaften, Attributen und Artikel-Filter. ',
	), $sortOrder++);
	if($catalogGroupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($catalogGroupId, 'AJAX_HANDLER', array(
			'FeatureSetAdmin'
		));
		$t_success &= $this->addAdminAccessGroupItem($catalogGroupId, 'CONTROLLER', array(
			'CategoriesApiV2',
			'CategoryIconsApiV2',
			'CategoryImagesApiV2',
			'NewAttributes',
			'ProductImagesApiV2',
			'ProductsApiV2',
			'ProductsLinksApiV2',
		));
		$t_success &= $this->addAdminAccessGroupItem($catalogGroupId, 'PAGE', array(
			'categories.php',
			'gm_feature_control.php',
			'gm_gmotion.php',
			'new_attributes.php',
			'products_attributes.php',
			'properties.php',
			'properties_combis.php',
			'specials.php',
		));
	}
	else
	{
		$t_success = false;
	}
	
	if($catalogGroupId !== false)
	{
		/**
		 * ---------- QuickEdit ----------
		 */
		$quickEditGroupId = $this->addAdminAccessGroup(array(1 => 'QuickEdit', 2 => 'QuickEdit'), array(
			1 => 'Allows the usage of the QuickEdit.',
			2 => 'Erlaubt die Nutzung von QuickEdit.',
		), $sortOrder++, $catalogGroupId);
		if($quickEditGroupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($quickEditGroupId, 'CONTROLLER', array(
				'QuickEditOverview',
				'QuickEditOverviewAjax',
				'QuickEditProductPropertiesAjax',
				'QuickEditSpecialPricesAjax',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Manufacturers ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Manufacturers', 2 => 'Hersteller'), array(
			1 => 'Allows the access to the manufacturers.',
			2 => 'Erlaubt den Zugriff auf die Hersteller.',
		), $sortOrder++, $catalogGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'Manufacturer',
				'ManufacturerAjax',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- VPE ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Packing Unit', 2 => 'Verpackungseinheiten'), array(
			1 => 'Allows the access to the packing units.',
			2 => 'Erlaubt den Zugriff auf die Verpackungseinheiten.',
		), $sortOrder++, $catalogGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'VPE',
				'VPEAjax',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Quantity Units ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Quantity Units', 2 => 'Mengeneinheiten'), array(
			1 => 'Allows the access to the quantity units.',
			2 => 'Erlaubt den Zugriff auf die Mengeneinheiten.',
		), $sortOrder++, $catalogGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'QuantityUnit',
				'QuantityUnitAjax',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- GX Customizer ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'GX-Customizer', 2 => 'GX-Customizer'), array(
			1 => 'Allows the access to the GX-Customizer.',
			2 => 'Erlaubt den Zugriff auf den GX-Customizer.',
		), $sortOrder++, $catalogGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_gprint.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Cross-selling Groups ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Cross-selling Groups', 2 => 'Cross-Marketing Gruppen'), array(
			1 => 'Allows the access to the cross-selling groups.',
			2 => 'Erlaubt den Zugriff auf die Cross-Marketing Gruppen.',
		), $sortOrder++, $catalogGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'cross_sell_groups.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Product Reviews ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Product Reviews', 2 => 'Kundenrezensionen'), array(
			1 => 'Allows the access to the product reviews.',
			2 => 'Erlaubt den Zugriff auf die Kundenrezensionen.',
		), $sortOrder++, $catalogGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'reviews.php',
			));
		}
		else
		{
			$t_success = false;
		}
	}
	
	/**
	 * ---------- Layout / Design ----------
	 */
	$layoutDesignGroupId = $this->addAdminAccessGroup(array(1 => 'Layout / Design', 2 => 'Darstellung'), array(
		1 => 'Allows the general usage of layout and design options.',
		2 => 'Erlaubt die generelle Nutzung von Darstellungsoptionen.',
	), $sortOrder++);
	if($layoutDesignGroupId !== false)
	{
		/**
		 * ---------- Layout / Design - Template Settings ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Template Settings', 2 => 'Template Einstellungen'), array(
			1 => 'Allows the management of the templates options and logos.',
			2 => 'Erlaubt die Verwaltung der Templates und Logos.',
		), $sortOrder++, $layoutDesignGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'TemplateConfiguration',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'template_configuration.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Layout / Design - Logo Manager ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Logo Manager', 2 => 'Logo Manager'), array(
			1 => 'Allows the usage of the logo manager.',
			2 => 'Erlaubt die Nutzung des Logo Managers.',
		), $sortOrder++, $layoutDesignGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_logo.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Layout / Design - Sliders ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Sliders', 2 => 'Sliders'), array(
			1 => 'Allows the management of the sliders.',
			2 => 'Erlaubt die Verwaltung von Slidern.',
		), $sortOrder++, $layoutDesignGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'SlidersDetails',
				'SlidersDetailsAjax',
				'SlidersOverview',
				'SlidersOverviewAjax',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_slider.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Layout / Design - Content Manager ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Content Manager', 2 => 'Content Manager'), array(
			1 => 'Allows the management of shop contents with the content manager.',
			2 => 'Erlaubt die Verwaltung der Shop Inhalte mit dem Content Manager.',
		), $sortOrder++, $layoutDesignGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'ContentManagerAjax',
				'ContentManagerElements',
				'ContentManagerPages',
				'ContentManagerProductContents',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'content_manager.php',
				'content_preview.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Layout / Design - Shop offline ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Shop offline', 2 => 'Shop offline'), array(
			1 => 'Allows the management of the "Shop offline" options.',
			2 => 'Erlaubt die Verwaltung der "Shop offline" Einstellungen.',
		), $sortOrder++, $layoutDesignGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'AJAX_HANDLER', array(
				'JobManager',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_offline.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Layout / Design - Banner Manager ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Banner Manager', 2 => 'Banner Manager'), array(
			1 => 'Allows the usage of the banner manager.',
			2 => 'Erlaubt die Nutzung des Banner Managers.',
		), $sortOrder++, $layoutDesignGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'banner_manager.php',
				'banner_statistics.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Layout / Design - Image processing ----------
		 */
		$imageProcessingGroupId = $this->addAdminAccessGroup(array(1 => 'Image Processing', 2 => 'Image Processing'),
		                                                     array(
			                                                     1 => 'Allows the usage of the image processing.',
			                                                     2 => 'Erlaubt die Nutzung des Image Processings.',
		                                                     ), $sortOrder++, $layoutDesignGroupId);
		if($imageProcessingGroupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($imageProcessingGroupId, 'CONTROLLER', array(
				'ImageProcessing',
			));
		}
		else
		{
			$t_success = false;
		}
	}
	
	/**
	 * ---------- Toolbox ----------
	 */
	$toolboxGroupId = $this->addAdminAccessGroup(array(1 => 'Toolbox', 2 => 'Toolbox'), array(
		1 => 'Allows the general usage of the tools in the toolbox.',
		2 => 'Erlaubt die generelle Nutzung der Tools der Toolbox.',
	), $sortOrder++);
	if($toolboxGroupId !== false)
	{
		/**
		 * ---------- Toolbox - Cache ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Cache', 2 => 'Cache'), array(
			1 => 'Allows the management of the caching system.',
			2 => 'Erlaubt die Verwaltung des Caching-Systems.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'clear_cache.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - Timer ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Timer', 2 => 'Timer'), array(
			1 => 'Allows the management of the timers.',
			2 => 'Erlaubt die Verwaltung der Timer.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'timer.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - OpenSearch ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'OpenSearch', 2 => 'OpenSearch'), array(
			1 => 'Allows the management of the OpenSearch plugin.',
			2 => 'Erlaubt die Verwaltung von OpenSearch-Plugins.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_opensearch.php',
				'gm_opensearch_action.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - Edit text phrases ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Edit text phrases', 2 => 'Texte bearbeiten'), array(
			1 => 'Allows the management of the shop text phrases',
			2 => 'Erlaubt die Verwaltung der Shop-Texte.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_lang_edit.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - Newsletter ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Newsletter', 2 => 'Newsletter'), array(
			1 => 'Allows the management of the newsletters.',
			2 => 'Erlaubt die Verwaltung der Newsletter.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'module_newsletter.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - Database Manager ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Database Manager', 2 => 'Datenbanksicherung'), array(
			1 => 'Allows the usage of the database manager.',
			2 => 'Erlaubt die Nutzung der Datenbanksicherung.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'backup.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - Backup files ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Backup files', 2 => 'Dateien sichern'), array(
			1 => 'Allows the access to the backup files.',
			2 => 'Erlaubt den Zugriff auf die Backup-Dateien.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_backup_files_zip.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - Logs ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Logs', 2 => 'Logs'), array(
			1 => 'Allows the management of the server logs.',
			2 => 'Erlaubt die Verwaltung der Serverlogs.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'show_logs.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - Server info ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Server info', 2 => 'Server Info'), array(
			1 => 'Allows the access to the server information.',
			2 => 'Erlaubt die Einsicht in die Serverinformationen.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'server_info.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - SQL ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'SQL', 2 => 'SQL'), array(
			1 => 'Allows the usage of the phpMiniAdmin.',
			2 => 'Erlaubt die Nutzung des phpMiniAdmins.',
		), $sortOrder++, $toolboxGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'EmbeddedModule/minisql',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'phpminiadmin.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - File Manager ----------
		 */
		$fileManagerGroupId = $this->addAdminAccessGroup(array(1 => 'File Manager', 2 => 'File Manager'), array(
			1 => 'Allows the usage of the FileManager.',
			2 => 'Erlaubt die Nutzung des FileManagers.',
		), $sortOrder++, $toolboxGroupId);
		if($fileManagerGroupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($fileManagerGroupId, 'CONTROLLER', array(
				'ResponsiveFileManager',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Toolbox - AutoUpdater ----------
		 */
		$fileManagerGroupId = $this->addAdminAccessGroup(array(1 => 'AutoUpdater', 2 => 'AutoUpdater'), array(
			1 => 'Allows the usage of the AutoUpdater.',
			2 => 'Erlaubt die Nutzung des AutoUpdaters.',
		), $sortOrder++, $toolboxGroupId);
		if($fileManagerGroupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($fileManagerGroupId, 'CONTROLLER', array(
				'AutoUpdater',
			));
		}
		else
		{
			$t_success = false;
		}
	}
	
	/**
	 * ---------- Modules ----------
	 */
	$modulesGroupId = $this->addAdminAccessGroup(array(1 => 'Modules', 2 => 'Module'), array(
		1 => 'Allows the general usage of modules from the module center.',
		2 => 'Erlaubt die generelle Nutzung von Modulen aus dem Modul-Center.',
	), $sortOrder++);
	if($modulesGroupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($modulesGroupId, 'CONTROLLER', array(
			'AfterbuyAjax',
			'Geschaeftskundenversand',
			'PackedDownload',
			'SharedShoppingCartConfiguration',
			'Shipcloud',
		));
		$t_success &= $this->addAdminAccessGroupItem($modulesGroupId, 'PAGE', array(
			'amazonadvpay.php',
			'brickfox.php',
			'ekomi.php',
			'findologic_config.php',
			'gm_janolaw.php',
			'gm_trusted_shop_id.php',
			'heidelpaygw.php',
			'hermes_collection.php',
			'hermes_config.php',
			'hermes_info.php',
			'hermes_list.php',
			'hermes_order.php',
			'interkurierConnect.php',
			'intraship.php',
			'klarna_config.php',
			'lettr_de.php',
			'magnalister.php',
			'mailbeez.php',
			'mediafinanz.php',
			'orders_iloxx.php',
			'payone_config.php',
			'payone_logs.php',
			'paypal.php',
			'print_intraship_label.php',
			'saferpay.php',
			'shopgate.php',
			'yatego.php',
			'yoochoose.php',
		));
		
		/**
		 * ---------- Module Center ----------
		 */
		$moduleCenterGroupId = $this->addAdminAccessGroup(array(1 => 'Module Center', 2 => 'Modul-Center'), array(
			1 => 'Allows the management of modules with the module center.',
			2 => 'Erlaubt die Verwaltung von Modulen über das Modul-Center.',
		), $sortOrder++, $modulesGroupId);
		if($moduleCenterGroupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($moduleCenterGroupId, 'CONTROLLER', array(
				'AfterbuyModuleCenterModule',
				'AmazonAdvPaymentsModuleCenterModule',
				'BrickfoxModuleCenterModule',
				'EkomiModuleCenterModule',
				'FindologicModuleCenterModule',
				'GeschaeftskundenversandModuleCenterModule',
				'GoogleAdwordConversionModuleCenterModule',
				'HeidelpayModuleCenterModule',
				'HermesModuleCenterModule',
				'IloxxModuleCenterModule',
				'IntrashipModuleCenterModule',
				'ItRechtModuleCenterModule',
				'JanolawModuleCenterModule',
				'KlarnaModuleCenterModule',
				'LettrModuleCenterModule',
				'MagnalisterModuleCenterModule',
				'MailbeezModuleCenterModule',
				'MediafinanzModuleCenterModule',
				'ModuleCenter',
				'NewsletterLinkModuleCenterModule',
				'ParcelshopfinderModuleCenterModule',
				'PayOneModuleCenterModule',
				'PaypalNGModuleCenterModule',
				'PostfinderModuleCenterModule',
				'ProductAttributesModuleCenterModule',
				'ProtectedShopsModuleCenterModule',
				'SharedShoppingCartModuleCenterModule',
				'ShipcloudModuleCenterModule',
				'TrustedShopsModuleCenterModule',
				'YategoModuleCenterModule',
				'YoochooseModuleCenterModule',
			));
		}
		else
		{
			$t_success = false;
		}
		
		$groupId = $this->addAdminAccessGroup(array(
			                                      1 => 'Shipping Methods, Payment Systems and Order Total',
			                                      2 => 'Versandartien, Zahlunhsweisen und Zusammenfassung'
		                                      ), array(
			                                      1 => 'Allows the management of shipping, payment and order total modules.',
			                                      2 => 'Erlaubt die Verwaltung von Versand-, Zahlungs- und Zusammenfassungs-Modulen.',
		                                      ), $sortOrder++, $modulesGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'modules.php',
			));
		}
		else
		{
			$t_success = false;
		}
	}
	else
	{
		$t_success = false;
	}
	
	/**
	 * ---------- Statistics ----------
	 */
	$groupId = $this->addAdminAccessGroup(array(1 => 'Statistics', 2 => 'Statistiken'), array(
		1 => 'Allows the access to the statistics.',
		2 => 'Erlaubt die Einsicht in die Statistiken.',
	), $sortOrder++);
	if($groupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
			'Dashboard/getStatisticBoxes',
		));
		$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
			'gm_counter.php',
			'gm_counter_action.php',
			'stats_campaigns.php',
			'stats_customers.php',
			'stats_products_purchased.php',
			'stats_products_viewed.php',
			'stats_sales_report.php',
			'stats_stock_warning.php',
			'whos_online.php',
		));
	}
	else
	{
		$t_success = false;
	}
	
	/**
	 * ---------- Import / Export ----------
	 */
	$importExportGroupId = $this->addAdminAccessGroup(array(1 => 'Import / Export', 2 => 'Import / Export'), array(
		1 => 'Allows the usage of imports and exports.',
		2 => 'Erlaubt die Nutzung des Import und Exports.',
	), $sortOrder++);
	if($importExportGroupId !== false)
	{
		/**
		 * ---------- Import / Export ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Products', 2 => 'Artikeldaten'), array(
			1 => 'Allows the export of products.',
			2 => 'Erlaubt den Export von Artikeldaten.',
		), $sortOrder++, $importExportGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'csv.php',
				'gm_product_export.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Import / Export ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Customer Export', 2 => 'Kundenexport'), array(
			1 => 'Allows the export of customers.',
			2 => 'Erlaubt den Export von Kundendaten.',
		), $sortOrder++, $importExportGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_module_export.php',
				'gm_module_part_export.php',
			));
		}
		else
		{
			$t_success = false;
		}
	}
	
	/**
	 * ---------- Vouchers ----------
	 */
	$groupId = $this->addAdminAccessGroup(array(1 => 'Vouchers', 2 => 'Gutscheine'), array(
		1 => 'Allows the managements of vouchers.',
		2 => 'Erlaubt die Verwaltung von Gutscheinen.',
	), $sortOrder++);
	if($groupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
			'coupon_admin.php',
			'gv_mail.php',
			'gv_queue.php',
			'gv_sent.php',
			'listcategories.php',
			'listproducts.php',
			'validcategories.php',
			'validproducts.php',
		));
	}
	else
	{
		$t_success = false;
	}
	
	/**
	 * ---------- Shop settings ----------
	 */
	$shopSettingsGroupId = $this->addAdminAccessGroup(array(1 => 'Shop Settings', 2 => 'Shop Einstellungen'), array(
		1 => 'Allows the general access to the shop options.',
		2 => 'Erlaubt den generellen Zugriff auf die Shop Einstellungen.',
	), $sortOrder++);
	if($shopSettingsGroupId !== false)
	{
		/**
		 * ---------- Shop settings ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'General settings', 2 => 'Allgemeine Einstellungen'), array(
			1 => 'Allows the management of the general options of the shop.',
			2 => 'Erlaubt die Verwaltung der allgemeinen Einstellungen des Shops.',
		), $sortOrder++, $shopSettingsGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'ShopInformationApiV2',
				'ShopKey',
				'VersionInfo',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'campaigns.php',
				'configuration.php',
				'gm_miscellaneous.php',
				'gm_pdf.php',
				'gm_pdf_action.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Law ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Rights', 2 => 'Rechtliches'), array(
			1 => 'Allows the management of the law options of the shop.',
			2 => 'Erlaubt die Verwaltung der rechtlichen Shop Einstellungen.',
		), $sortOrder++, $shopSettingsGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'Laws',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_laws.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Shipping status ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Shipping status', 2 => 'Lieferstatus'), array(
			1 => 'Allows the management of the shipping status of the shop.',
			2 => 'Erlaubt die Verwaltung der Lieferstatus des Shops.',
		), $sortOrder++, $shopSettingsGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'shipping_status.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Parcel services ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Parcel services', 2 => 'Paketdienste'), array(
			1 => 'Allows the management of the parcel services of the shop.',
			2 => 'Erlaubt die Verwaltung der Paketdienste des Shops.',
		), $sortOrder++, $shopSettingsGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'parcel_services.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Start IDs ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Start IDs', 2 => 'Nummernkreise'), array(
			1 => 'Allows the management of the starting ids of the shop.',
			2 => 'Erlaubt die Verwaltung der Start IDs des Shops.',
		), $sortOrder++, $shopSettingsGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_id_starts.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- SEO ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'SEO', 2 => 'SEO'), array(
			1 => 'Allows the management of the SEO options of the shop.',
			2 => 'Erlaubt die Verwaltung der SEO-Einstellungen des Shops.',
		), $sortOrder++, $shopSettingsGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'StaticSeoUrl',
				'StaticSeoUrlAjax',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_analytics.php',
				'gm_meta.php',
				'gm_meta_action.php',
				'gm_seo_boost.php',
				'gm_sitemap.php',
				'gm_sitemap_creator.php',
				'robots_download.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Zone & Tax ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Zone & Tax', 2 => 'Zonen & Steuern'), array(
			1 => 'Allows the management of zones and taxes.',
			2 => 'Erlaubt die Verwaltung von Zonen und Steuern.',
		), $sortOrder++, $shopSettingsGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'CountriesApiV2',
				'TaxClassesApiV2',
				'TaxRatesApiV2',
				'TaxZonesApiV2',
				'ZonesApiV2',
			));
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'countries.php',
				'currencies.php',
				'geo_zones.php',
				'languages.php',
				'tax_classes.php',
				'tax_rates.php',
				'zones.php',
			));
		}
		else
		{
			$t_success = false;
		}
		
		/**
		 * ---------- Security ----------
		 */
		$groupId = $this->addAdminAccessGroup(array(1 => 'Security', 2 => 'Sicherheit'), array(
			1 => 'Allows the management of the security options of the shop.',
			2 => 'Erlaubt die Verwaltung der Sicherheits-Einstellungen des Shops.',
		), $sortOrder++, $shopSettingsGroupId);
		if($groupId !== false)
		{
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'gm_security.php',
			));
		}
		else
		{
			$t_success = false;
		}
	}
	
	/**
	 * ---------- Hub ----------
	 */
	$hubGroupId = $this->addAdminAccessGroup(array(1 => 'Hub', 2 => 'Hub'), array(
		1 => 'Allows the management of the Gambio Hub.',
		2 => 'Erlaubt die Verwaltung des Gambio Hubs.',
	), $sortOrder++);
	if($hubGroupId !== false)
	{
		$t_success &= $this->addAdminAccessGroupItem($hubGroupId, 'CONTROLLER', array(
			'HubConfiguration',
			'HubConfigurationAjax',
		));
	}
	else
	{
		$t_success = false;
	}
}
elseif(!$this->table_column_exists('admin_access_groups', 'protected'))
{
	$query     = 'ALTER TABLE admin_access_groups ADD COLUMN `protected` SMALLINT UNSIGNED NOT NULL DEFAULT "0"';
	$t_success &= $this->query($query, true);
}

$t_check = $this->table_exists('admin_access_permissions');
if(!$t_check)
{
	$t_query   = "
		CREATE TABLE `admin_access_permissions` (
			`admin_access_role_id` INT(11) UNSIGNED NOT NULL,
			`admin_access_group_id` INT(11) UNSIGNED NOT NULL,
			`reading_granted` SMALLINT UNSIGNED NOT NULL DEFAULT '0',
			`writing_granted` SMALLINT UNSIGNED NOT NULL DEFAULT '0',
			`deleting_granted` SMALLINT UNSIGNED NOT NULL DEFAULT '0',
			PRIMARY KEY (`admin_access_group_id`, `admin_access_role_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
	";
	$t_success &= $this->query($t_query);
}

$t_check = $this->table_exists('admin_access_role_descriptions');
if(!$t_check)
{
	$t_query   = "
		CREATE TABLE `admin_access_role_descriptions` (
			`admin_access_role_id` INT(11) UNSIGNED NOT NULL DEFAULT '1',
			`language_id` int(11) NOT NULL DEFAULT '1',
			`name` varchar(255) NOT NULL DEFAULT '',
			`description` text NOT NULL,
			PRIMARY KEY (`admin_access_role_id`, `language_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
	";
	$t_success &= $this->query($t_query);
}

$t_check        = $this->table_exists('admin_access_roles');
$rolesSortOrder = 1;
if(!$t_check)
{
	$t_query   = "
		CREATE TABLE `admin_access_roles` (
			`admin_access_role_id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
			`sort_order` INT(11) UNSIGNED NOT NULL DEFAULT '0',
			`deleting_unknown_group_granted` SMALLINT UNSIGNED NOT NULL DEFAULT '1',
			`reading_unknown_group_granted` SMALLINT UNSIGNED NOT NULL DEFAULT '1',
			`writing_unknown_group_granted` SMALLINT UNSIGNED NOT NULL DEFAULT '1',
			`protected` SMALLINT UNSIGNED NOT NULL DEFAULT '0',
			PRIMARY KEY (`admin_access_role_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
	";
	$t_success &= $this->query($t_query);
	
	$roleId = $this->addAdminAccessRole(array(
		                                    1 => 'Super Administrator',
		                                    2 => 'Super-Administrator'
	                                    ), array(
		                                    1 => 'Superior administrator role, that has every permission. This role should neither be deleted nor changed.',
		                                    2 => 'Übergeordnete Administrator-Rolle, die jegliche Berechtigung besitzt. Diese Rolle sollte weder gelöscht noch geändert werden.',
	                                    ), $rolesSortOrder++);
	if($roleId !== false)
	{
		$amountOfSqlErrors = count($this->get_sql_errors());
		$t_query           = "
			INSERT INTO `admin_access_permissions`
				(`admin_access_role_id`, `admin_access_group_id`, `reading_granted`, `writing_granted`, `deleting_granted`)
			SELECT " . $roleId . ", `admin_access_group_id`, 1, 1, 1 FROM `admin_access_groups`;
		";
		$this->query($t_query);
		$t_success &= $amountOfSqlErrors === count($this->get_sql_errors());
	}
	else
	{
		$t_success = false;
	}
}
elseif(!$this->table_column_exists('admin_access_roles', 'protected'))
{
	$query     = 'ALTER TABLE admin_access_roles ADD COLUMN `protected` SMALLINT UNSIGNED NOT NULL DEFAULT "0"';
	$t_success &= $this->query($query, true);
}

$t_check = $this->table_exists('admin_access_users');
if(!$t_check)
{
	$t_query   = "
		CREATE TABLE `admin_access_users` (
			`customer_id` INT(11) NOT NULL DEFAULT '0',
			`admin_access_role_id` INT(11) UNSIGNED NOT NULL DEFAULT '0',
			PRIMARY KEY (`admin_access_role_id`, `customer_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
	";
	$t_success &= $this->query($t_query);
}

/**
 * ---------- Import old admin_access configurations into new AdminAccessService ----------
 */

if(!$adminAccessAlreadyImported)
{
	/**
	 * @var $columns mysqli_result
	 */
	$columns           = $this->query('SHOW COLUMNS FROM `admin_access`', true);
	$groupMappingArray = array();
	
	while($column = $columns->fetch_assoc())
	{
		if($column['Field'] === 'customers_id')
		{
			continue;
		}
		
		$groupId = $this->getAdminAccessGroupIdByIdentifier('PAGE', $column['Field'] . '.php');
		if($groupId === false || $groupId === null)
		{
			continue;
		}
		
		$groupMappingArray[$column['Field']] = $groupId;
	}
	
	/**
	 * @var $oldAdminAccessTable array
	 */
	$oldAdminAccessTable = $this->query('SELECT * FROM `admin_access`');
	
	foreach($oldAdminAccessTable as $adminPermissions)
	{
		$customerId = $adminPermissions['customers_id'];
		if($customerId === 'groups')
		{
			continue;
		}
		
		$customerId = (int)$customerId;
		if($customerId <= 1)
		{
			continue;
		}
		
		$adminName = $this->query('SELECT `customers_firstname` AS `firstname`, `customers_lastname` AS `lastname` FROM `customers` WHERE `customers_id` = "'
		                          . $customerId . '"');
		if($adminName === false || $adminName === null)
		{
			continue;
		}
		
		$adminName = $adminName[0];
		
		$roleId = $this->addAdminAccessRole(array(
			                                    1 => 'Admin role for ' . $adminName['firstname'] . ' '
			                                         . $adminName['lastname'],
			                                    2 => 'Admin Rolle für ' . $adminName['firstname'] . ' '
			                                         . $adminName['lastname']
		                                    ), array(
			                                    1 => 'This role was generated automatically during the update process.',
			                                    2 => 'Diese Rolle wurde automatisch während des Update-Prozesses generiert.'
		                                    ), $rolesSortOrder++);
		if($roleId === false)
		{
			continue;
		}
		
		foreach($groupMappingArray as $field => $groupId)
		{
			$permissionGranted = (int)$adminPermissions[$field] === 1;
			
			if($permissionGranted)
			{
				$this->grantAdminAccessPermission($roleId, $groupId);
				
				if($field === 'categories')
				{
					$this->grantAdminAccessPermission($roleId, $quickEditGroupId);
					$this->grantAdminAccessPermission($roleId, $imageProcessingGroupId);
				}
				
				if($field === 'modules')
				{
					$this->grantAdminAccessPermission($roleId, $moduleCenterGroupId);
					$this->grantAdminAccessPermission($roleId, $hubGroupId);
				}
				
				if($field === 'phpminiadmin')
				{
					$this->grantAdminAccessPermission($roleId, $fileManagerGroupId);
				}
			}
		}
		
		$this->grantAdminAccessPermission($roleId, $apiGroupId);
		
		$this->addAdminAccessRoleToUserByCustomerId($roleId, $customerId);
	}
}

/**
 * Add Kosovo's district as Serbian district if not exist.
 */
$zonesNames = array(
	'Kosovsko-Pristina okrug' => '30',
	'Kosovsko-Peja okrug'     => '31',
	'Kosovsko-Prizren okrug'  => '32',
	'Kosovsko-Ferizaj okrug'  => '33',
	'Kosovsko-Gjilan okrug'   => '34',
	'Kosovsko-Gjakova okrug'  => '35',
);

$t_check = $this->query('SELECT * FROM `zones` WHERE `zone_country_id` = 240 ', true);

$databaseZone = array();

while($row = $t_check->fetch_assoc())
{
	$databaseZone[$row['zone_name']] = $row['zone_code'];
}

$result = array_diff_assoc($zonesNames, $databaseZone);

if(!empty($result))
{
	
	foreach($result as $zoneName => $zoneCode)
	{
		$query     = "INSERT INTO `zones` (`zone_country_id`, `zone_code`, `zone_name`) VALUE('240', '$zoneCode', '$zoneName')";
		$t_success &= $this->query($query, true);
	}
}

/*
 * Uninstall old PayPal modules
 */
$paymentModulesResult = $this->query("SELECT * FROM `configuration` WHERE `configuration_key` = 'MODULE_PAYMENT_INSTALLED'",
                                     true);
if($paymentModulesResult->num_rows > 0)
{
	$removeModules          = array('paypalng.php', 'paypalgambio_alt.php', 'paypal.php', 'paypalexpress.php');
	$modulePaymentInstalled = 'fail';
	foreach($paymentModulesResult as $paymentModulesResultRow)
	{
		$modulePaymentInstalled = $paymentModulesResultRow['configuration_value'];
	}
	$paymentModules        = explode(';', $modulePaymentInstalled);
	$paymentModulesCleaned = array_diff($paymentModules, $removeModules);
	$updatePaymentsQuery   = "UPDATE `configuration` SET `configuration_value` = ':payment_installed' WHERE `configuration_key` = 'MODULE_PAYMENT_INSTALLED'";
	$updatePaymentsQuery   = strtr($updatePaymentsQuery,
	                               array(':payment_installed' => implode(';', $paymentModulesCleaned)));
	$updatePaymentsSuccess = $this->query($updatePaymentsQuery, true) === false ? 0 : 1;
	$t_success             &= $updatePaymentsSuccess;
	$deactivateQuery       = "DELETE FROM `configuration` WHERE `configuration_key` = 'MODULE_PAYMENT_:code_STATUS'";
	foreach($removeModules as $moduleFileName)
	{
		$moduleCode            = strtoupper(str_replace('.php', '', $moduleFileName));
		$deactivateModuleQuery = strtr($deactivateQuery, array(':code' => $moduleCode));
		$this->query($deactivateModuleQuery);
	}
}

/*
 * Make Module Center AdminAccess group a subgroup of Modules
 */
$moduleCenterGroupId = $this->getAdminAccessGroupIdByIdentifier('CONTROLLER', 'ModuleCenter');
$modulesGroupId      = $this->getAdminAccessGroupIdByIdentifier('CONTROLLER', 'Geschaeftskundenversand');

$check = $this->query('SELECT * FROM `admin_access_groups` WHERE `admin_access_group_id` = "' . $moduleCenterGroupId
                      . '";', true);
if($check->num_rows !== 0 && $modulesGroupId > 0)
{
	$query     = 'UPDATE `admin_access_groups` SET `parent_id` = "' . $modulesGroupId
	             . '" WHERE `admin_access_group_id` = "' . $moduleCenterGroupId . '";';
	$t_success &= $this->query($query, true);
}

/*
 * Move AdminAccess group item "UserConfiguration" to admin web ui
 */
$adminWebUiGroupId = $this->getAdminAccessGroupIdByIdentifier('PAGE', 'admin.php');
if($adminWebUiGroupId > 0)
{
	$query     = 'DELETE FROM `admin_access_group_items` WHERE `identifier` = "UserConfiguration" && `type` = "CONTROLLER";';
	$t_success &= $this->query($query, true);
	
	$query     = 'INSERT INTO `admin_access_group_items` (`admin_access_group_id`, `identifier`,  `type`) VALUES ('
	             . $adminWebUiGroupId . ', "UserConfiguration", "CONTROLLER");';
	$t_success &= $this->query($query, true);
}

/*
 * Create new admin access group "Shipping Methods, Payment Systems and Order Total"
 */
$check = $this->query('SELECT * FROM `admin_access_group_descriptions` WHERE `name` = "Shipping Methods, Payment Systems and Order Total";',
                      true);
if($check->num_rows === 0)
{
	$modulesGroupId = $this->getAdminAccessGroupIdByIdentifier('CONTROLLER', 'Geschaeftskundenversand');
	if($modulesGroupId > 0)
	{
		$groupId = $this->addAdminAccessGroup(array(
			                                      1 => 'Shipping Methods, Payment Systems and Order Total',
			                                      2 => 'Versandartien, Zahlunhsweisen und Zusammenfassung'
		                                      ), array(
			                                      1 => 'Allows the management of shipping, payment and order total modules.',
			                                      2 => 'Erlaubt die Verwaltung von Versand-, Zahlungs- und Zusammenfassungs-Modulen.',
		                                      ), 100, $modulesGroupId);
		if($groupId !== false)
		{
			$query     = 'DELETE FROM `admin_access_group_items` WHERE `identifier` = "modules.php" && `type` = "PAGE";';
			$t_success &= $this->query($query, true);
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'PAGE', array(
				'modules.php',
			));
		}
		else
		{
			$t_success = false;
		}
	}
}

# Add admin access group item "CustomerGroup"
$customerStatusGroupId = $this->getAdminAccessGroupIdByIdentifier('PAGE', 'customers_status.php');
if($customerStatusGroupId !== false)
{
	$t_success &= $this->addAdminAccessGroupItem($customerStatusGroupId, 'CONTROLLER', array(
		'CustomerGroup',
	));
}

/*
 * Create new admin access group "AutoUpdater"
 */
$check = $this->query('SELECT * FROM `admin_access_group_descriptions` WHERE `name` = "AutoUpdater";', true);
if($check->num_rows === 0)
{
	$toolboxGroupId = $this->getAdminAccessGroupIdByName('Toolbox');
	if($toolboxGroupId > 0)
	{
		$groupId = $this->addAdminAccessGroup(array(
			                                      1 => 'AutoUpdater',
			                                      2 => 'AutoUpdater'
		                                      ), array(
			                                      1 => 'Allows the usage of the AutoUpdater.',
			                                      2 => 'Erlaubt die Nutzung des AutoUpdaters.',
		                                      ), 100, $toolboxGroupId);
		if($groupId !== false)
		{
			$query     = 'DELETE FROM `admin_access_group_items` WHERE `identifier` = "AutoUpdater" && `type` = "CONTROLLER";';
			$t_success &= $this->query($query, true);
			$t_success &= $this->addAdminAccessGroupItem($groupId, 'CONTROLLER', array(
				'AutoUpdater',
			));
		}
		else
		{
			$t_success = false;
		}
	}
}

/*
 * Reassign several admin access group items to groups
 */
$reassignments = array(
	array(
		'type'       => 'PAGE',
		'identifier' => 'csv.php',
		'newGroupId' => 44,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_product_export.php',
		'newGroupId' => 44,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_module_export.php',
		'newGroupId' => 45,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_module_part_export.php',
		'newGroupId' => 45,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'coupon_admin.php',
		'newGroupId' => 46,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gv_mail.php',
		'newGroupId' => 46,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gv_queue.php',
		'newGroupId' => 46,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gv_sent.php',
		'newGroupId' => 46,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'listcategories.php',
		'newGroupId' => 46,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'listproducts.php',
		'newGroupId' => 46,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'validcategories.php',
		'newGroupId' => 46,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'validproducts.php',
		'newGroupId' => 46,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'ShopInformationApiV2',
		'newGroupId' => 48,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'ShopKey',
		'newGroupId' => 48,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'VersionInfo',
		'newGroupId' => 48,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'campaigns.php',
		'newGroupId' => 48,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'configuration.php',
		'newGroupId' => 48,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_miscellaneous.php',
		'newGroupId' => 48,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_pdf.php',
		'newGroupId' => 48,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_pdf_action.php',
		'newGroupId' => 48,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'Laws',
		'newGroupId' => 49,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_laws.php',
		'newGroupId' => 49,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'shipping_status.php',
		'newGroupId' => 50,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'parcel_services.php',
		'newGroupId' => 51,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_id_starts.php',
		'newGroupId' => 52,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'StaticSeoUrl',
		'newGroupId' => 53,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'StaticSeoUrlAjax',
		'newGroupId' => 53,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_analytics.php',
		'newGroupId' => 53,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_meta.php',
		'newGroupId' => 53,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_meta_action.php',
		'newGroupId' => 53,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_seo_boost.php',
		'newGroupId' => 53,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_sitemap.php',
		'newGroupId' => 53,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_sitemap_creator.php',
		'newGroupId' => 53,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'robots_download.php',
		'newGroupId' => 53,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'CountriesApiV2',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'TaxClassesApiV2',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'TaxRatesApiV2',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'TaxZonesApiV2',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'ZonesApiV2',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'countries.php',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'currencies.php',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'geo_zones.php',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'languages.php',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'tax_classes.php',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'tax_rates.php',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'zones.php',
		'newGroupId' => 54,
	),
	array(
		'type'       => 'PAGE',
		'identifier' => 'gm_security.php',
		'newGroupId' => 55,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'HubConfiguration',
		'newGroupId' => 56,
	),
	array(
		'type'       => 'CONTROLLER',
		'identifier' => 'HubConfigurationAjax',
		'newGroupId' => 56,
	),
);
foreach($reassignments as $reassignment)
{
	$query     = sprintf('DELETE FROM `admin_access_group_items` WHERE `identifier` = "%s" && `type` = "%s";',
	                     $reassignment['identifier'], $reassignment['type']);
	$t_success &= $this->query($query, true);
	$t_success &= $this->addAdminAccessGroupItem($reassignment['newGroupId'], $reassignment['type'],
	                                             $reassignment['identifier']);
}

$t_check = $this->query('SELECT `gm_configuration_id` FROM `gm_configuration` WHERE `gm_key` = "MODULE_CENTER_GAMBIOOVISTO_INSTALLED"',
    true);
if ($t_check->num_rows === 0) {
    $this->query('INSERT INTO `gm_configuration` (`gm_key`, `gm_value`) VALUES ("MODULE_CENTER_GAMBIOOVISTO_INSTALLED", 1)');
    
    $t_check = $this->query('SELECT `key` FROM `configuration_storage` WHERE `key` = "modules/GambioOvisto/active"',
        true);
    if($t_check->num_rows === 0)
    {
        $this->query('INSERT INTO `configuration_storage` (`key`, `value`) VALUES ("modules/GambioOvisto/active", 1)');
    }
}